import math
import random
import heapq
from Tkinter import *

class Node:
	def __init__(self, name, xPos, yPos):
		self.name = name
		self.pos = [xPos, yPos]
		self.disp = [0.0, 0.0]
	
	def __str__(self):
		return str(self.name)+" "+ str(self.pos)
		
	def __repr__(self):
		return str(self.name)+" "+str(self.pos)
	
class Graph:
	def __init__(self, w, l):
		self.nodes = []
		self.edges = {}
		self.width = w
		self.length = l
		self.k = 0
		self.t = w/5

	def addNode(self, node):
		self.nodes.append(node)
		self.edges[node] = []
		
	def addEdge(self, weight, node1, node2):
		self.edges[node1].append((weight, node2, node1))
		self.edges[node2].append((weight, node1, node2))
	
	def calculateK(self):
		area = float(self.width * self.length)
		num = float(len(self.nodes))
		divided = area/num
		self.k = math.sqrt(divided)
		return self.k
		
	def iterate(self):
		for node in self.nodes:
			repVec = self.computeRepulsive(node)
			attVec = self.computeAttractive(node)
			node.disp[0] += repVec[0]+attVec[0]
			node.disp[1] += repVec[1]+attVec[1]
		#for node in self.nodes:
			lenDiff = math.sqrt(node.disp[0]**2 + node.disp[1]**2)+.001
			node.pos[0] += (node.disp[0]/lenDiff)*min(abs(node.disp[0]), self.t)
			node.pos[1] += (node.disp[1]/lenDiff)*min(abs(node.disp[1]), self.t)
			node.pos = [min(self.width-5, max(5, node.pos[0])), min(self.length-5, max(5, node.pos[1]))]
			
		self.cool()
		
	def computeRepulsive(self, node):
		repVec = [0.0, 0.0]
		for other in self.nodes:
			if (node != other):
				delta = (node.pos[0]-other.pos[0], node.pos[1]-other.pos[1])
				lenDelta = math.sqrt(delta[0]**2+delta[1]**2)+.001
				force = self.forceRepulse(lenDelta)
				repVec[0] += (delta[0]/lenDelta)*force
				repVec[1] += (delta[1]/lenDelta)*force
		return repVec
		
	def forceRepulse(self, z):
		x = (2*self.k)-z
		if x > 0:
			return (self.k**2)/z
		else:
			return 0
		
	def computeAttractive(self, node):
		attVec = [0.0, 0.0]
		for weight, node1, node2 in self.edges[node]:
			delta = (node1.pos[0]-node2.pos[0], node1.pos[1]-node2.pos[1])
			lenDelta = math.sqrt(delta[0]**2+delta[1]**2)+.001
			force = self.forceAttract(lenDelta)
			attVec[0] += (delta[0]/lenDelta)*force
			attVec[1] += (delta[1]/lenDelta)*force
		return attVec
		
	def forceAttract(self, x):
		return (x**2)/(self.k)
		
	def cool(self):
		self.t = self.t-(self.t/10)
		
	def resize(self):
		avg = 0.0
		for node, edgeList in self.edges.iteritems():
			for edge in edgeList:
				avg += edge[0]
		for node in self.nodes:
			attVec = [0.0, 0.0]
			for weight, node1, node2 in self.edges[node]:
				delta = (node1.pos[0]-node2.pos[0], node1.pos[1]-node2.pos[1])
				lenDelta = math.sqrt(delta[0]**2+delta[1]**2)+.001
				force = self.forceAttract(lenDelta/avg)
				attVec[0] += (delta[0]/lenDelta)*force
				attVec[1] += (delta[1]/lenDelta)*force
				lenDiff = math.sqrt(attVec[0]**2 + attVec[1]**2)+.001
				node.pos[0] += (attVec[0]/lenDiff)*abs(attVec[0])*20
				node.pos[1] += (attVec[1]/lenDiff)*abs(attVec[1])*20
				node.pos = [min(self.width-5, max(5, node.pos[0])), min(self.length-5, max(5, node.pos[1]))]

	def primsMST(self):
		tree = {}
		nodeList = [self.nodes[0]]
		candidates = []
		for edge in self.edges[self.nodes[0]]:
			candidates.append(edge)
		tree[self.nodes[0]] = []
		for i in range(len(self.nodes)-1):		
			newEdge = self.findEdge(candidates, nodeList)
			tree[newEdge[1]] = []
			tree[newEdge[2]].append((newEdge[0], newEdge[1], newEdge[2]))
			tree[newEdge[1]].append((newEdge[0], newEdge[2], newEdge[1]))
			nodeList.append(newEdge[1])
			for weight, edge1, edge2 in self.edges[newEdge[1]]:
				if edge1 not in nodeList:
					candidates.append((weight, edge1, edge2))
		self.edges = tree
			
				
	def findEdge(self, candidates, usedNodes):
		while candidates:
			candidate = max(candidates)
			if candidate[1] not in usedNodes:
				candidates.remove(candidate)
				return candidate
			candidates.remove(candidate)
		print "No MST Exists"
		return 
		
		

		
random.seed(3)		
g = Graph(500,500)
for i in range(25):
	g.addNode(Node(i,random.randint(0,g.width),random.randint(0,g.length)))
for node in g.nodes:
	for node2 in g.nodes:
		if node != node2:
			g.addEdge(random.randint(1,100), node, node2)
print 'start'
g.primsMST()
print 'finished'
for node, edgeList in g.edges.iteritems():
	for edge in edgeList:
		print edge
g.calculateK()

canvas = Canvas(width=500, height=500, bg='white')  
canvas.pack(expand=YES, fill=BOTH)                           

'''for node in g.nodes:
	canvas.create_oval(node.pos[0]-2, node.pos[1]-2, node.pos[0]+2, node.pos[1]+2, width=1, fill='red')
for node, edgeList in g.edges.iteritems():
	for edge in edgeList:
		node1 = edge[1]
		node2 = edge[2]
		canvas.create_line(node1.pos[0], node1.pos[1], node2.pos[0], node2.pos[1], fill='red')
'''
for i in range(100):
	g.iterate()                       
'''	for node in g.nodes:
		canvas.create_oval(node.pos[0]-2, node.pos[1]-2, node.pos[0]+2, node.pos[1]+2, width=1, fill='blue')
	for node, edgeList in g.edges.iteritems():
		for edge in edgeList:
			node1 = edge[1]
			node2 = edge[2]
			canvas.create_line(node1.pos[0], node1.pos[1], node2.pos[0], node2.pos[1], fill='blue')
'''	
#g.resize()

for node in g.nodes:
	canvas.create_oval(node.pos[0]-2, node.pos[1]-2, node.pos[0]+2, node.pos[1]+2, width=1, fill='blue')
		
for node, edgeList in g.edges.iteritems():
	for edge in edgeList:
		node1 = edge[1]
		node2 = edge[2]
		canvas.create_line(node1.pos[0], node1.pos[1], node2.pos[0], node2.pos[1], fill='blue')

mainloop()
