import math
import random
from sets import Set
from Tkinter import *

class Node:
	def __init__(self, name, xPos, yPos):
		self.name = name
		self.pos = [xPos, yPos]
		self.disp = [0.0, 0.0]
	
	def __str__(self):
		return str(self.name)+" "+ str(self.pos)
		
	def __repr__(self):
		return str(self.name)+" "+str(self.pos)
	
class Graph:
	def __init__(self, w, l):	
		self.nodes = []
		self.edges = []
		self.width = w
		self.length = l
		self.k = 0
		self.t = w/10.0

	def addNode(self, node):
		self.nodes.append(node)
		
	def addEdge(self, weight, node1, node2):
		self.edges.append((weight, node1, node2))
	
	def calculateK(self):
		area = float(self.width * self.length)
		num = float(len(self.nodes))
		divided = area/num
		self.k = math.sqrt(divided)
		return self.k
		
	def forceAttract(self, x):
		#print "attr x2", x**2, "k", self.k
		return (x**2)/self.k
		
	def forceRepulse(self, z):
		x = (2*self.k)-z
		if x > 0:
		#print "rep k2", self.k**2, "z", z
			return (self.k**2)/z
		else:
			return 0
		
	def iterate(self):
		for node in self.nodes:
			node.disp = self.computeRepulsive(node)
		for edge in self.edges:
			attVec = self.computeAttractive(edge)
			edge[1].disp = [edge[1].disp[0]-attVec[0], edge[1].disp[1]-attVec[1]]
			edge[2].disp = [edge[2].disp[0]+attVec[0], edge[2].disp[1]+attVec[1]]
		for node in self.nodes:
			lenDiff = math.sqrt(node.disp[0]**2 + node.disp[1]**2)+.001
			node.pos[0] += (node.disp[0]/lenDiff)*min(abs(node.disp[0]), self.t)
			node.pos[1] += (node.disp[1]/lenDiff)*min(abs(node.disp[1]), self.t)
			node.pos = [min(self.width-10, max(10, node.pos[0])), min(self.length-10, max(10, node.pos[1]))]
			
		self.cool()
		
	def computeRepulsive(self, node):
		repVec = (0.0, 0.0)
		for other in self.nodes:
			if (node != other):
				delta = (node.pos[0]-other.pos[0], node.pos[1]-other.pos[1])
				lenDelta = math.sqrt(delta[0]**2+delta[1]**2)+.001
				force = self.forceRepulse(lenDelta)
				repVec = [repVec[0]+(delta[0]/lenDelta)*force, repVec[1]+(delta[1]/lenDelta)*force]
		return repVec
		
	def computeAttractive(self, edge):
		node1 = edge[1]
		node2 = edge[2]
		delta = (node1.pos[0]-node2.pos[0], node1.pos[1]-node2.pos[1])
		lenDelta = math.sqrt(delta[0]**2+delta[1]**2)+.001
		force = self.forceAttract(lenDelta)
		attVec = [(delta[0]/lenDelta)*force, (delta[1]/lenDelta)*force]
		return attVec
			
		
	def cool(self):
		self.t = self.t
		
	def mst(self):
		tree = []
		maxEdge = max(self.edges)
		nodeList = set([maxEdge[1], maxEdge[2]])
		tree.append(maxEdge)
		self.edges.remove(maxEdge)
		sets = []
		while len(nodeList) != len(self.nodes):

			maxEdge = max(self.edges)
			node1 = maxEdge[1]
			node2 = maxEdge[2]
			if (node1 not in nodeList) and (node2 not in nodeList):
				added = True
				for i in sets:
					if (node1 in i) and (node2 in i):
						added = False
					elif (node1 in i) ^ (node2 in i):
						sets.remove(i)
						for j in sets:
							if i != j:
								if (node1 in j) ^ (node2 in j):
									i = i.union(j)
									sets.remove(j)
						i.add(node1)
						i.add(node2)
						sets.append(i)
						tree.append(maxEdge)
						added = False
				if added:
					aSet = set([node1, node2])
					sets.append(aSet)
					tree.append(maxEdge)
			elif (node1 in nodeList) ^ (node2 in nodeList):
				tree.append(maxEdge)
				nodeList.add(node1)
				nodeList.add(node2)
				for aSet in sets:
					if (node1 in aSet) or (node2 in aSet):
						nodeList = nodeList.union(aSet)
						sets.remove(aSet)
					
			self.edges.remove(maxEdge)
		self.edges = tree

		
random.seed(3)		
g = Graph(1300,800)
for i in range(15):
	g.addNode(Node(i,random.randint(0,g.width),random.randint(0,g.length)))
for node in g.nodes:
	for node2 in g.nodes:
		if node != node2:
			g.addEdge(random.randint(1,100), node, node2)
#for node in g.nodes:
#	print node
print 'start'
g.mst()
print 'finished'
print g.edges
g.calculateK()
print g.k
for i in range(200):
	g.iterate()

	'''canvas = Canvas(width=500, height=500, bg='white')  
	canvas.pack(expand=YES, fill=BOTH)                

	for node in g.nodes:
		canvas.create_oval(node.pos[0]-1, node.pos[1]-1, node.pos[0]+1, node.pos[1]+1, width=1, fill='blue')
		print node
	for edge in g.edges:
		node1 = edge[1]
		node2 = edge[2]
		canvas.create_line(node1.pos[0], node1.pos[1], node2.pos[0], node2.pos[1])

	mainloop()'''
	
#for node in g.nodes:
#	print node

canvas = Canvas(width=1300, height=800, bg='white')  
canvas.pack(expand=YES, fill=BOTH)                

for node in g.nodes:
	canvas.create_oval(node.pos[0]-1, node.pos[1]-1, node.pos[0]+1, node.pos[1]+1, width=1, fill='blue')
	canvas.create_text(node.pos[0], node.pos[1]-5, text=node.name, state='hidden') 
	#print node
for edge in g.edges:
	node1 = edge[1]
	node2 = edge[2]
	canvas.create_line(node1.pos[0], node1.pos[1], node2.pos[0], node2.pos[1])

mainloop()
